"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const list_1 = require("cli-ux/lib/list");
const chalk = require("chalk");
function buildUsage(command) {
    if (command.usage)
        return command.usage.trim();
    let cmd = command.id;
    if (!command.parse.args)
        return cmd.trim();
    let args = command.parse.args.map(renderArg);
    return `${cmd} ${args.join(' ')}`.trim();
}
function renderArg(arg) {
    let name = arg.name.toUpperCase();
    if (arg.required !== false && arg.optional !== true)
        return `${name}`;
    else
        return `[${name}]`;
}
class Help {
    constructor(config) {
        this.config = config;
    }
    command(cmd) {
        let flags = Object.entries(cmd.parse.flags || {}).filter(([, flag]) => !flag.hidden);
        let args = (cmd.parse.args || []).filter(a => !a.hidden);
        let hasFlags = flags.length ? ` ${chalk.blue('[flags]')}` : '';
        let usage = `${chalk.bold('Usage:')} ${this.config.bin} ${buildUsage(cmd)}${hasFlags}\n`;
        return [
            usage,
            cmd.description ? `\n${chalk.bold(cmd.description.trim())}\n` : '',
            this.renderAliases(cmd.aliases),
            this.renderArgs(args),
            this.renderFlags(flags),
            cmd.help ? `\n${cmd.help.trim()}\n` : '',
        ].join('');
    }
    commandLine(cmd) {
        return [buildUsage(cmd), cmd.description ? chalk.dim(cmd.description) : null];
    }
    renderAliases(aliases) {
        if (!aliases || !aliases.length)
            return '';
        let a = aliases.map(a => `  $ ${this.config.bin} ${a}`).join('\n');
        return `\n${chalk.blue('Aliases:')}\n${a}\n`;
    }
    renderArgs(args) {
        if (!args.find(f => !!f.description))
            return '';
        return ('\n' +
            list_1.renderList(args.map(a => {
                return [a.name.toUpperCase(), a.description ? chalk.dim(a.description) : null];
            })) +
            '\n');
    }
    renderFlags(flags) {
        if (!flags.length)
            return '';
        flags.sort((a, b) => {
            if (a[1].char && !b[1].char)
                return -1;
            if (b[1].char && !a[1].char)
                return 1;
            if (a[0] < b[0])
                return -1;
            return b[0] < a[0] ? 1 : 0;
        });
        return (`\n${chalk.blue('Flags:')}\n` +
            list_1.renderList(flags.map(([name, f]) => {
                let label = [];
                if (f.char)
                    label.push(`-${f.char}`);
                if (name)
                    label.push(` --${name}`);
                let usage = f.type === 'option' ? ` ${name.toUpperCase()}` : '';
                let description = f.description || '';
                if (f.required)
                    description = `(required) ${description}`;
                return [` ${label.join(',').trim()}` + usage, description ? chalk.dim(description) : null];
            })) +
            '\n');
    }
}
exports.Help = Help;
